@@ -85,7 +85,7 @@ int info=TRUE,nogrids=0,nomeshes=0,activemesh=0;
 #if EXE_MODE
 static int PartitionMesh(int nofile) 
 {
-  /* Partititioning related stuff */
+  /* Partitioning related stuff */
 
   int noopt = 0;
 
@@ -422,7 +422,7 @@ static int ImportMeshDefinition(int inmethod,int nofile,char *filename,int *nogr
     visited = TRUE;
   }
 
-  /* Native format of ElmerGrid gets specieal treatment */
+  /* Native format of ElmerGrid gets special treatment */
   switch (inmethod) {
     
   case 1: 
@@ -494,7 +494,8 @@ static int ImportMeshDefinition(int inmethod,int nofile,char *filename,int *nogr
   case 14:
     data[nofile].dim = 3; /* default dim 3 with gmsh*/
     eg.dim = 3;
-    errorstat = LoadGmshInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],info);
+    errorstat = LoadGmshInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],
+			      eg.multidim,eg.dim,info);
     break;
 
    
@@ -850,7 +851,7 @@ int main(int argc, char *argv[])
   static Real mergeeps;
   long ii;
 
-  if(info) printf("\nStarting program Elmergrid\n");
+  if(info) printf("\nStarting program Elmergrid, compiled on %s\n", __DATE__ );
   
   InitParameters(&eg);
   grids = (struct GridType*)malloc((size_t) (MAXCASES)*sizeof(struct GridType));     
@@ -899,7 +900,7 @@ int main(int argc, char *argv[])
   if(info) printf("\nElmergrid creating and manipulating meshes:\n");
   ManipulateMeshDefinition(inmethod,outmethod,relh);
 
-  /* Partititioning related stuff */
+  /* Partitioning related stuff */
   for(k=0;k<nomeshes;k++) 
     PartitionMesh(nofile); 
 
