@@ -40,7 +40,19 @@
 
 #include <QtGui>
 #include <iostream>
+#include <QPushButton>
+#include <QMainWindow>
+#include <QMdiArea>
+#include <QMdiSubWindow>
+#include <QFileDialog>
+#include <QMessageBox>
+
+#if WITH_QT6
+#define QRegExp QRegularExpression
+#endif
+
 #include "edfeditor.h"
+//#include "MainWindow.h"
 
 using namespace std;
 
@@ -49,15 +61,15 @@ using namespace std;
 EdfEditor::EdfEditor(QWidget *parent)
   : QWidget(parent)
 {
-  addIcon = QIcon(":/icons/list-add.png");
-  removeIcon = QIcon(":/icons/list-remove.png");
-  collapseIcon = QIcon(":/icons/arrow-up.png");
-  expandIcon = QIcon(":/icons/arrow-down.png");
-  openIcon = QIcon(":/icons/document-open.png");
-  appendIcon = QIcon(":/icons/tab-new-background.png"); // todo
-  saveAsIcon = QIcon(":/icons/document-save.png");
-  applyIcon = QIcon(":/icons/dialog-close.png");
-  previewIcon = QIcon(":/icons/document-preview.png");
+  addIcon = QIcon::fromTheme("list-add");
+  removeIcon = QIcon::fromTheme("list-remove");
+  collapseIcon = QIcon::fromTheme("triangle-up");
+  expandIcon = QIcon::fromTheme("triangle-down");
+  openIcon = QIcon::fromTheme("document-open");
+  appendIcon = QIcon::fromTheme("tab-new"); // todo
+  saveAsIcon = QIcon::fromTheme("document-save");
+  applyIcon = QIcon::fromTheme("dialog-error-round");
+  previewIcon = QIcon::fromTheme("edit-find");
 
   lastActiveItem = NULL;
   ctrlPressed = false;
@@ -156,12 +168,33 @@ EdfEditor::EdfEditor(QWidget *parent)
   dynamicEditorIC = new DynamicEditor;
 
   setWindowIcon(QIcon(":/icons/Mesh3D.png"));
+  
+    
+#ifdef __APPLE__DONTGO_HERE_TODO
+  defaultDir = ((MainWindow*)paernt->homePath ;
+#else
+  defaultDir =
+    QCoreApplication::applicationDirPath() + "/../share/ElmerGUI";
+
+  QString elmerGuiHome = QString(getenv("ELMERGUI_HOME"));
+
+  if (!elmerGuiHome.isEmpty())
+    defaultDir = elmerGuiHome;
+
+  defaultDir.replace('\\', '/');
+#endif
+
+  // Commented out as restoring defaultDir is not so useful
+  //defaultDir = ((MainWindow*) parent)->settings_value("defaultDir/edfEditor", defaultDir).toString();
+
 }
 
+
 // dtor...
 //----------------------------------------------------------------------------
 EdfEditor::~EdfEditor()
 {
+
 }
 
 // Min window size...
@@ -362,15 +395,26 @@ void EdfEditor::updateElement(QTreeWidgetItem *item, int column)
   QRegExp expression(pattern);
   QString qs = item->text(1).trimmed();
 
+#if WITH_QT6
+  QRegularExpressionMatch rem;
+  int index = qs.indexOf(expression, 0, &rem);
+#else
   int index = qs.indexOf(expression);
+#endif
 
   QString parsedString = "";
   if(index < 0)
     parsedString = qs;
 
   while(index >= 0) {
+#if WITH_QT6
+    QString currentMatch = rem.captured(0);	
+    int length = currentMatch.length();
+#else
     int length = expression.matchedLength();
     QString currentMatch = qs.mid(index, length);
+#endif
+
     QStringList currentList = currentMatch.split("=");
     QString name = currentList.at(0);
     QString value = currentList.at(1);
@@ -382,7 +426,13 @@ void EdfEditor::updateElement(QTreeWidgetItem *item, int column)
     parsedString.append(name + "=\"" + value + "\" ");
 
     element.setAttribute(name.trimmed(), value.trimmed());
+	
+#if WITH_QT6
+    index = qs.indexOf(expression, index + length, &rem);
+#else
     index = qs.indexOf(expression, index + length);
+#endif
+
   }
 
   // update display with parsed attributes
@@ -485,7 +535,7 @@ void EdfEditor::saveAsButtonClicked()
   QString fileName;
 
   fileName = QFileDialog::getSaveFileName(this,
-                 tr("Save definitions"), "", tr("EDF (*.xml)") );
+                 tr("Save definitions"), defaultDir, tr("EDF (*.xml)") );
 
   if(fileName.isEmpty())
     return;
@@ -497,6 +547,10 @@ void EdfEditor::saveAsButtonClicked()
   file.open(QIODevice::WriteOnly);
   QTextStream out(&file);
   elmerDefs->save(out, indent);
+  
+  // Commented out as restoring defaultDir is not so useful
+  // QFileInfo info(file);
+  // defaultDir = info.dir().absolutePath();  
 }
 
 
@@ -524,7 +578,7 @@ void EdfEditor::openButtonClicked()
   QString fileName;
 
   fileName = QFileDialog::getOpenFileName(this, 
-	      tr("Open definitions"), "", tr("EDF (*.xml)") );
+	      tr("Open definitions"), defaultDir, tr("EDF (*.xml)") );
 
   if(fileName.isEmpty())
     return;
@@ -559,9 +613,12 @@ void EdfEditor::openButtonClicked()
   setupEditor(elmerDefs);
 
   edfTree->setCurrentItem(NULL);
+  
+  // Commented out as restoring defaultDir is not so useful
+  //QFileInfo info(file);
+  //defaultDir = info.dir().absolutePath();  
 }
 
-
 // Append...
 //----------------------------------------------------------------------------
 void EdfEditor::appendButtonClicked()
@@ -569,7 +626,7 @@ void EdfEditor::appendButtonClicked()
   QString fileName;
 
   fileName = QFileDialog::getOpenFileName(this, 
-	      tr("Open definitions"), "", tr("EDF (*.xml)") );
+	      tr("Open definitions"), defaultDir, tr("EDF (*.xml)") );
 
   if(fileName.isEmpty())
     return;
@@ -614,6 +671,10 @@ void EdfEditor::appendButtonClicked()
   setupEditor(elmerDefs);
 
   edfTree->setCurrentItem(NULL);
+  
+  // Commented out as restoring defaultDir is not so useful
+  //QFileInfo info(file);
+  //defaultDir = info.dir().absolutePath();  
 }
 
 
@@ -767,4 +828,8 @@ bool EdfEditor::appendFrom(QString path)
   edfTree->setCurrentItem(NULL);
 
   return true;
-}
\ No newline at end of file
+}
+
+QString EdfEditor::defaultEdfDir(){
+  return defaultDir; 
+}
