@@ -27,7 +27,7 @@
  *                                                                           *
  *****************************************************************************
  *                                                                           *
- *  Authors: Mikko Lyly, Juha Ruokolainen and Peter Rback                   *
+ *  Authors: Mikko Lyly, Juha Ruokolainen and Peter Råback                   *
  *  Email:   Juha.Ruokolainen@csc.fi                                         *
  *  Web:     http://www.csc.fi/elmer                                         *
  *  Address: CSC - IT Center for Science Ltd.                                 *
@@ -157,14 +157,20 @@ bool list_t::isVisible(void) const
 
 // Construct glWidget...
 //-----------------------------------------------------------------------------
+#if WITH_QT6
+GLWidget::GLWidget(QWidget *parent)
+  : QOpenGLWidget(parent)
+#else
 GLWidget::GLWidget(QWidget *parent)
   : QGLWidget(parent)
+#endif
 {
   backgroundColor = Qt::white;
   surfaceColor = Qt::cyan;
   edgeColor = Qt::green;
   surfaceMeshColor = Qt::black;
   sharpEdgeColor = Qt::black;
+  selectionColor = Qt::red;
 
   stateOrtho = false;
   stateFlatShade = true;
@@ -194,10 +200,6 @@ GLWidget::GLWidget(QWidget *parent)
   helpers = new Helpers;
   meshutils = new Meshutils;
 
-  ctrlPressed = false;
-  shiftPressed = false;
-  altPressed = false;
-
   // Coordinate axis:
   quadric_axis = gluNewQuadric();
 
@@ -267,6 +269,11 @@ bool GLWidget::hasMesh(void) const
 void GLWidget::initializeGL()
 {
   cout << "Initialize GL" << endl;
+  
+#if WITH_QT6
+  initializeOpenGLFunctions();
+#endif
+
   cout << "Vendor: " << glGetString(GL_VENDOR) << endl;
   cout << "Renderer: " << glGetString(GL_RENDERER) << endl;
   cout << "Version: " << glGetString(GL_VERSION) << endl;
@@ -312,7 +319,11 @@ void GLWidget::initializeGL()
 
   glEnable(GL_NORMALIZE);
 
+#if WITH_QT6
+  glClearColor( backgroundColor.redF(), backgroundColor.greenF(), backgroundColor.blueF(), backgroundColor.alphaF());
+#else
   qglClearColor(backgroundColor);
+#endif
 
   glEnable(GL_TEXTURE_2D);
 }
@@ -323,6 +334,9 @@ void GLWidget::initializeGL()
 //-----------------------------------------------------------------------------
 void GLWidget::paintGL()
 {
+#if WITH_QT6
+  glClearColor( backgroundColor.redF(), backgroundColor.greenF(), backgroundColor.blueF(), backgroundColor.alphaF());
+#endif
   float xabs[3], xrel[3];
 
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -640,15 +654,6 @@ void GLWidget::focusInEvent(QFocusEvent *event)
 //-----------------------------------------------------------------------------
 void GLWidget::keyPressEvent(QKeyEvent *event)
 {
-  if(event->key() == Qt::Key_Control)
-    ctrlPressed = true;
-
-  if(event->key() == Qt::Key_Shift)
-    shiftPressed = true;
-
-  if((event->key() == Qt::Key_Alt) || (event->key() == Qt::Key_AltGr))
-    altPressed = true;
-
   if(event->key() == Qt::Key_Escape)
     emit(escPressed());
 }
@@ -658,36 +663,49 @@ void GLWidget::keyPressEvent(QKeyEvent *event)
 //-----------------------------------------------------------------------------
 void GLWidget::keyReleaseEvent(QKeyEvent *event)
 {
-  if(event->key() == Qt::Key_Control)
-    ctrlPressed = false;
-
-  if(event->key() == Qt::Key_Shift)
-    shiftPressed = false;
-
-  if(event->key() == Qt::Key_Alt)
-    altPressed = false;
 }
 
 
 
-// Mouse button clicked...
+// Mouse button pressed...
 //-----------------------------------------------------------------------------
 void GLWidget::mousePressEvent(QMouseEvent *event)
 {
   lastPos = event->pos();
+  lastPressPos = event->pos();
   setFocus();  // for tracing keyboard events
 }
 
-
+// Mouse button released...
+//-----------------------------------------------------------------------------
+void GLWidget::mouseReleaseEvent(QMouseEvent *event)
+{
+  if((event->button() == Qt::RightButton) && (event->pos() == lastPressPos)) {
+#if WITH_QT6
+    ((MainWindow*)parent())->showContextMenu(event->globalPosition().toPoint());
+#else
+    ((MainWindow*)parent())->showContextMenu(event->globalPos());
+#endif	
+  }
+}
 
 // Mouse wheel rotates...
 //-----------------------------------------------------------------------------
 void GLWidget::wheelEvent(QWheelEvent *event)
 {
+#if WITH_QT6
+  makeCurrent();
+  double s = exp((double)(event->angleDelta().y())*0.001);
+#else
   double s = exp((double)(event->delta())*0.001);
+#endif
   glScaled(s, s, s);
   updateGL();
+#if WITH_QT6  
+  lastPos = event->position().toPoint();
+#else
   lastPos = event->pos();
+#endif
   getMatrix();
 }
 
@@ -697,23 +715,39 @@ void GLWidget::wheelEvent(QWheelEvent *event)
 //-----------------------------------------------------------------------------
 void GLWidget::mouseMoveEvent(QMouseEvent *event)
 {
+#if WITH_QT6
+  makeCurrent();
+#endif
+
   GLint viewport[4];
   glGetIntegerv(GL_VIEWPORT, viewport);
 
+#if WITH_QT6
+  int dx = event->position().x() - lastPos.x();
+  int dy = event->position().y() - lastPos.y();
+#else
   int dx = event->x() - lastPos.x();
   int dy = event->y() - lastPos.y();
+#endif
 
   dy = -dy;
   
-  if (((event->buttons() & Qt::LeftButton) && 
-       (event->buttons() & Qt::MidButton)) ) {
+  if (
+#if WITH_QT6
+  ((event->buttons() & Qt::LeftButton) && (event->buttons() & Qt::MiddleButton))
+#else
+  ((event->buttons() & Qt::LeftButton) && (event->buttons() & Qt::MidButton))
+#endif
+        ||
+    event->buttons() == Qt::RightButton  // added for easy scaling
+       ) {
 
     // Scale:
     double s = exp(dy*0.01);
     glScaled(s, s, s);
     updateGL();
 
-  } else if (event->buttons() & Qt::LeftButton) {
+  } else if (event->buttons() == Qt::LeftButton) {
     
     // Rotation:
     double ax = -(double)dy;
@@ -727,7 +761,16 @@ void GLWidget::mouseMoveEvent(QMouseEvent *event)
     glRotated(s, bx, by, bz);
     updateGL();
 
-  } else if (event->buttons() & Qt::MidButton) {
+  } else if (
+#if WITH_QT6
+  (event->buttons() == Qt::MiddleButton)
+#else
+  (event->buttons() == Qt::MidButton)
+#endif
+  
+  ||
+  (event->buttons() ==  (Qt::LeftButton | Qt::RightButton)) // added for 2 button mouse
+    ){
 
     // Translation:
     double s = 2.0/(double)(viewport[3]+1);
@@ -750,9 +793,30 @@ void GLWidget::mouseMoveEvent(QMouseEvent *event)
 //-----------------------------------------------------------------------------
 void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
 {
+#if WITH_QT6
+  makeCurrent();
+#endif
   if(getLists() == 0) 
     return;
-
+  
+  /*
+  To avoid segmentation fault in MSYS2 environment, compass, numbers and indexes are hidden.
+  These will be restored at the end of this function. Do not return before restoring these.
+  */
+  bool prevStateDrawCoordinates = stateDrawCoordinates;
+  bool prevStateDrawSurfaceNumbers = stateDrawSurfaceNumbers;
+  bool prevStateDrawEdgeNumbers = stateDrawEdgeNumbers;
+  bool prevStateDrawNodeNumbers = stateDrawNodeNumbers;
+  bool prevStateDrawBoundaryIndex = stateDrawBoundaryIndex;
+  bool prevStateDrawBodyIndex = stateDrawBodyIndex;  
+  stateDrawCoordinates = false;
+  stateDrawSurfaceNumbers = false;
+  stateDrawEdgeNumbers = false;
+  stateDrawNodeNumbers = false;
+  stateDrawBoundaryIndex = false;
+  stateDrawBodyIndex = false;
+  
+  
   static list_t dummylist;
   static GLuint buffer[1024];
   const int bufferSize = sizeof(buffer)/sizeof(GLuint);
@@ -764,6 +828,9 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
   GLint i, j;
 
   updateGL();
+#if WITH_QT6
+  makeCurrent();
+#endif
   
   glSelectBuffer(bufferSize, buffer);
   glRenderMode(GL_SELECT);
@@ -774,9 +841,14 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
   glGetIntegerv(GL_VIEWPORT, viewport);
   glGetDoublev(GL_PROJECTION_MATRIX, projection);
   glLoadIdentity();
-  
+
+#if WITH_QT6
+  GLdouble x = event->position().x();
+  GLdouble y = (double)viewport[3]-event->position().y()-1;
+#else
   GLdouble x = event->x();
   GLdouble y = (double)viewport[3]-event->y()-1;
+#endif  
   
   GLdouble deltaX = 3.0;
   GLdouble deltaY = 3.0;
@@ -786,8 +858,18 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
   
   glMatrixMode(GL_MODELVIEW);
 
+
+  /*This is to avoid segmentation fault in Linux with old hardware*/
+  setMeshVisibility(false, false, false);
+  
+#if WITH_QT6
+  paintGL();
+#else
   updateGL();
-  // paintGL();
+#endif
+
+  /*Again, this is to avoid segmentation fault in Linux with old hardware and to suppress blinking*/
+  setMeshVisibility(stateDrawSurfaceMesh, stateDrawVolumeMesh, stateDrawSharpEdges); 
 
   hits = glRenderMode(GL_RENDER);
   
@@ -831,15 +913,29 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
     list_t *l = getList(nearest);
 
     // skip sharp edge lists
-    if(l->getType() == SHARPEDGELIST) 
-      return;
+   if(l->getType() == SHARPEDGELIST) {
+
+	  /* 
+	  Restoration of view settings. These were adjusted at the beginning of this function.
+	  */
+	  stateDrawCoordinates = prevStateDrawCoordinates;
+	  stateDrawSurfaceNumbers = prevStateDrawSurfaceNumbers;
+	  stateDrawEdgeNumbers = prevStateDrawEdgeNumbers;
+	  stateDrawNodeNumbers = prevStateDrawNodeNumbers;
+	  stateDrawBoundaryIndex = prevStateDrawBoundaryIndex;
+	  stateDrawBodyIndex = prevStateDrawBodyIndex;  
+	  
+
+      updateGL();
+	  return;
+	}
     
     // substitute surfacemeshlist with the parent surfacelist:
     if(l->getType() == SURFACEMESHLIST)
       l = getList(l->getParent());
 
     // if not ctrl pressed, rebuild all selected lists except this one:
-    if(!ctrlPressed) {
+    if(!(event->modifiers() & Qt::ControlModifier)) {
       for(i = 0; i < getLists(); i++) {
 	list_t *l2 = getList(i);
 	if(l2->isSelected() && (l2->getIndex() != l->getIndex())) {
@@ -872,7 +968,7 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
     // Highlight current selection:
     if(l->getType() == SURFACELIST) {
       if(l->isSelected()) {
-	l->setObject(generateSurfaceList(l->getIndex(), Qt::red)); // red
+	l->setObject(generateSurfaceList(l->getIndex(), selectionColor)); // red
       } else {
 	l->setObject(generateSurfaceList(l->getIndex(), surfaceColor)); // cyan
       }
@@ -884,7 +980,7 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
 
     } else if(l->getType() == EDGELIST) {
       if(l->isSelected()) {
-	l->setObject(generateEdgeList(l->getIndex(), Qt::red)); // red
+	l->setObject(generateEdgeList(l->getIndex(), selectionColor)); // red
       } else {
 	l->setObject(generateEdgeList(l->getIndex(), edgeColor)); // green
       }
@@ -897,7 +993,7 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
     // body selection:
     //----------------
     currentlySelectedBody = -1;
-    if(shiftPressed || bodyEditActive) {
+    if( (event->modifiers() & Qt::ShiftModifier) || bodyEditActive) {
 
       // determine the max bulk index
       int MAX_BULK_INDEX = -1;
@@ -929,7 +1025,7 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
       MAX_BULK_INDEX++;
       if(MAX_BULK_INDEX == 0) {
 	cout << "Error in body selection: "
-	  "There are no legal body indiced from which to choose" << endl;
+	  "There are no legal body indices from which to choose" << endl;
 	cout.flush();
 	goto body_selection_finished;
       }
@@ -985,18 +1081,23 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
 	  count++;
 	  found = i;
 	}
+//	cout << i << ": tmp1,2= " << tmp1[i] << "," << tmp2[i] << endl; 
       }
       
       if((count == 1) && (found >= 0))
 	currentlySelectedBody = found;
-      
+    else if((count > 1)){
+	  int m = mostVisibleBody(MAX_BULK_INDEX, tmp1);
+	  if(m >=0) currentlySelectedBody = m;
+	}
+     
       delete [] tmp1;
       delete [] tmp2;
     }
   body_selection_finished:
     
     // Emit result to mainwindow:
-    emit(signalBoundarySelected(l));
+    emit(signalBoundarySelected(l, event->modifiers()));
 
   } else {
 
@@ -1004,10 +1105,21 @@ void GLWidget::mouseDoubleClickEvent(QMouseEvent *event)
     dummylist.setNature(-1);
     dummylist.setType(-1);
     dummylist.setIndex(-1);
-    emit(signalBoundarySelected(&dummylist));
+    emit(signalBoundarySelected(&dummylist, event->modifiers()));
 
   }
 
+  /* 
+  Restoration of view settings. These were adjusted at the beginning of this function.
+  */
+  stateDrawCoordinates = prevStateDrawCoordinates;
+  stateDrawSurfaceNumbers = prevStateDrawSurfaceNumbers;
+  stateDrawEdgeNumbers = prevStateDrawEdgeNumbers;
+  stateDrawNodeNumbers = prevStateDrawNodeNumbers;
+  stateDrawBoundaryIndex = prevStateDrawBoundaryIndex;
+  stateDrawBodyIndex = prevStateDrawBodyIndex;  
+  
+  
   updateGL();
 }
 
@@ -1027,6 +1139,9 @@ void GLWidget::getMatrix()
 //-----------------------------------------------------------------------------
 void GLWidget::rebuildLists()
 {
+#if WITH_QT6
+  makeCurrent();
+#endif
   double *bb = mesh->boundingBox();
   
   drawTranslate[0] = bb[6]; // x-center
@@ -1059,7 +1174,7 @@ void GLWidget::rebuildSurfaceLists()
      {
        glDeleteLists( l->getObject(), 1 );
        if(l->isSelected()) {
- 	 l->setObject(generateSurfaceList(l->getIndex(), Qt::red)); // red
+ 	 l->setObject(generateSurfaceList(l->getIndex(), selectionColor)); // red
        } else {
  	 l->setObject(generateSurfaceList(l->getIndex(), surfaceColor)); // cyan
        }
@@ -1078,7 +1193,7 @@ void GLWidget::rebuildEdgeLists()
      {
        glDeleteLists( l->getObject(), 1 );
        if(l->isSelected()) {
- 	 l->setObject(generateEdgeList(l->getIndex(), Qt::red)); // red
+ 	 l->setObject(generateEdgeList(l->getIndex(), selectionColor)); // red
        } else {
  	 l->setObject(generateEdgeList(l->getIndex(), edgeColor)); // green
        }
@@ -1092,6 +1207,10 @@ void GLWidget::rebuildEdgeLists()
 //-----------------------------------------------------------------------------
 GLuint GLWidget::makeLists()
 {
+#if WITH_QT6
+  makeCurrent();
+#endif
+
   int i;
   list_t *l;
 
@@ -1111,7 +1230,7 @@ GLuint GLWidget::makeLists()
   // - A list of sharp edges will always be drawn (even if it is empty)
   //---------------------------------------------------------------------------
   
-  // Simultaneously, populate hash for mapping body & boundary incides:
+  // Simultaneously, populate hash for mapping body & boundary indices:
   //--------------------------------------------------------------------
   boundaryMap.clear();
   bodyMap.clear();
@@ -1821,6 +1940,9 @@ void GLWidget::drawCoordinates()
 
 bool GLWidget::toggleCoordinates()
 {
+#if WITH_QT6
+  makeCurrent();
+#endif
   stateDrawCoordinates = !stateDrawCoordinates;
   updateGL();
   return stateDrawCoordinates;
@@ -1833,16 +1955,21 @@ void GLWidget::drawBgImage()
   GLint viewport[4];
 
   if(!bgTexture) {
-#if WITH_QT5
+#if WITH_QT5 || WITH_QT6
     cout << "Bind texture " << string(bgImageFileName.toLatin1()) << "... ";
+	QOpenGLTexture texture(QImage(bgImageFileName).mirrored());
+	bgSizeX = texture.width();
+    bgSizeY = texture.height();
+    bgTexture = texture.textureId();
+    cout << "done" << endl;
 #else
     cout << "Bind texture " << string(bgImageFileName.toAscii()) << "... ";
-#endif
     QPixmap pixmap(bgImageFileName);
     bgSizeX = pixmap.width();
     bgSizeY = pixmap.height();
     bgTexture = bindTexture(pixmap, GL_TEXTURE_2D);
     cout << "done" << endl;
+#endif
   }
   
   if(!bgTexture) {
@@ -1940,3 +2067,174 @@ void GLWidget::indexColors(int *c, int i)
   indexColors(tmp, i);
   for (int j = 0; j < 3; j++) c[j] = int(tmp[j]*255 + 0.5);
 }
+
+
+void GLWidget::setMeshVisibility(bool stateDrawSurfaceMesh, bool stateDrawVolumeMesh, bool stateDrawSharpEdges){
+/*
+  This function is used in mouseDoubleClickEvent(mouseEvent event) to avoid segmentation fault observed Linux
+  environment with old hardware.
+*/
+
+  mesh_t *mesh = getMesh();
+  int lists = getLists();
+
+  if (mesh == NULL) {
+    return;
+  }
+
+
+  for (int i = 0; i < lists; i++) {
+    list_t *l = getList(i);
+	int type = l->getType();
+    if (type == SURFACEMESHLIST) {
+      l->setVisible(stateDrawSurfaceMesh);
+
+      // do not set visible if the parent surface list is hidden
+      int p = l->getParent();
+      if (p >= 0) {
+        list_t *lp = getList(p);
+        if (!lp->isVisible())
+          l->setVisible(false);
+      }
+    }else if (type == VOLUMEMESHLIST) {
+      l->setVisible(stateDrawVolumeMesh);
+	}else if (type == SHARPEDGELIST) {
+      l->setVisible(stateDrawSharpEdges);
+	}
+  }
+}
+
+int GLWidget::mostVisibleBody(int n, bool* tmp1){
+/*
+This function is called in GLWidget::mouseDoubleClickEvent(QMouseEvent *event) to
+identify the most visible body when the double-clicked surface is shared by multiple bodies.
+This is a public function to be called from ObjectBrowser. 
+*/
+
+  long *nElement = new long[n];
+  long *nVisibleElement = new long[n];
+  for(int i = 0; i < n; i++){
+    nElement[i] = 0;
+    nVisibleElement[i] = 0;
+  }
+
+  for(int i = 0; i < getLists(); i++) {
+    list_t *l2 = getList(i);
+	if(l2->getNature() == PDE_BOUNDARY && l2->getType() == SURFACELIST) {
+      for(int j = 0; j < mesh->getSurfaces(); j++) {
+	    surface_t *surf = mesh->getSurface(j);
+	    if(surf->getIndex() == l2->getIndex()) { 
+	      for(int k = 0; k < surf->getElements(); k++) {
+		    int l = surf->getElementIndex(k);
+ 		    if(l < 0) 
+		      break;
+		    element_t *elem = mesh->getElement(l);
+		    if((elem->getIndex() < 0) || (elem->getIndex() >= n))
+		      break;
+		  	nElement[elem->getIndex()]++;
+		    if(l2->isVisible())nVisibleElement[elem->getIndex()]++;
+          }
+		}
+      }
+    }
+  }
+  
+  double max = -1.0;
+  int selected = -1;
+  double visibility = -2.0;
+  for(int i = 0; i < n; i++){
+    if(tmp1[i] && nElement[i] > 0){
+    visibility = ((double) nVisibleElement[i]) / nElement[i];
+//cout << i << " visibility=" << visibility << " (" << nVisibleElement[i] << "/" << nElement[i] << ")" << endl; 
+      if(visibility > max){
+		max = visibility;
+		selected = i;
+	  }
+    }
+  }
+  delete[] nElement;
+  delete[] nVisibleElement;
+//cout << "selected: " << selected << endl;  
+  return selected;
+}
+
+#if WITH_QT6
+void GLWidget::updateGL()
+{
+	update();
+}
+
+
+
+/*
+ Reference http://stackoverflow.com/questions/28216001/how-to-render-text-with-qopenglwidget/33674071#33674071
+*/
+void GLWidget::renderText(double x, double y, double z, const QString & str, const QFont & font/* = QFont()*/, int listBase /*= 2000*/)
+{
+	int width = this->width();
+    int height = this->height();
+
+    GLdouble model[4][4], proj[4][4];
+    GLint view[4];
+    glGetDoublev(GL_MODELVIEW_MATRIX, &model[0][0]);
+    glGetDoublev(GL_PROJECTION_MATRIX, &proj[0][0]);
+    glGetIntegerv(GL_VIEWPORT, &view[0]);
+    GLdouble textPosX = 0, textPosY = 0, textPosZ = 0;
+
+    project(x, y, z, 
+                &model[0][0], &proj[0][0], &view[0],
+                &textPosX, &textPosY, &textPosZ);
+
+    textPosY = height - textPosY; // y is inverted
+
+    QPainter painter(this);
+    painter.setPen(Qt::black);
+    painter.setFont(font);
+    painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
+    painter.drawText(textPosX, textPosY, str); // z = pointT4.z + distOverOp / 4
+    painter.end();
+}
+
+inline GLint GLWidget::project(GLdouble objx, GLdouble objy, GLdouble objz,
+    const GLdouble model[16], const GLdouble proj[16],
+    const GLint viewport[4],
+    GLdouble * winx, GLdouble * winy, GLdouble * winz)
+{
+    GLdouble in[4], out[4];
+
+    in[0] = objx;
+    in[1] = objy;
+    in[2] = objz;
+    in[3] = 1.0;
+    transformPoint(out, model, in);
+    transformPoint(in, proj, out);
+
+    if (in[3] == 0.0)
+        return GL_FALSE;
+
+    in[0] /= in[3];
+    in[1] /= in[3];
+    in[2] /= in[3];
+
+    *winx = viewport[0] + (1 + in[0]) * viewport[2] / 2;
+    *winy = viewport[1] + (1 + in[1]) * viewport[3] / 2;
+
+    *winz = (1 + in[2]) / 2;
+    return GL_TRUE;
+}
+
+inline void GLWidget::transformPoint(GLdouble out[4], const GLdouble m[16], const GLdouble in[4])
+{
+#define M(row,col)  m[col*4+row]
+    out[0] =
+        M(0, 0) * in[0] + M(0, 1) * in[1] + M(0, 2) * in[2] + M(0, 3) * in[3];
+    out[1] =
+        M(1, 0) * in[0] + M(1, 1) * in[1] + M(1, 2) * in[2] + M(1, 3) * in[3];
+    out[2] =
+        M(2, 0) * in[0] + M(2, 1) * in[1] + M(2, 2) * in[2] + M(2, 3) * in[3];
+    out[3] =
+        M(3, 0) * in[0] + M(3, 1) * in[1] + M(3, 2) * in[2] + M(3, 3) * in[3];
+#undef M
+}
+
+#endif
