@@ -39,6 +39,7 @@
  *****************************************************************************/
 
 #include <QtGui>
+#include <QColorDialog>
 #include <iostream>
 #include "epmesh.h"
 #include "vtkpost.h"
@@ -78,6 +79,10 @@ StreamLine::StreamLine(QWidget *parent)
   connect(ui.keepLimits, SIGNAL(stateChanged(int)), this, SLOT(keepLimitsSlot(int)));
 
   setWindowIcon(QIcon(":/icons/Mesh3D.png"));
+
+  ui.cancelButton->setIcon(QIcon::fromTheme("dialog-error-round"));
+  ui.applyButton->setIcon(QIcon::fromTheme("view-refresh"));  
+  ui.okButton->setIcon(QIcon::fromTheme("dialog-accept"));
   
 #if VTK_MAJOR_VERSION >= 8
   ui.propagLabel->setText(QString("Max propagation:"));
@@ -87,6 +92,8 @@ StreamLine::StreamLine(QWidget *parent)
   ui.threads->setValue(2000);
 #endif
 
+  setNullColor(Qt::blue);
+  connect(ui.nullColorButton, SIGNAL(clicked()), this, SLOT(nullColorButtonClicked()));
 }
 
 StreamLine::~StreamLine()
@@ -147,6 +154,23 @@ void StreamLine::colorSelectionChanged(int newIndex)
     ui.minVal->setText(QString::number(sf->minVal));
     ui.maxVal->setText(QString::number(sf->maxVal));
   }
+  if(ui.colorCombo->currentIndex() == 0 ){ // i.e. Null field
+    ui.nullColorLabel->show();
+    ui.nullColorButton->show();
+	ui.minVal->setEnabled(false);
+	ui.maxVal->setEnabled(false);
+	ui.minLabel->setEnabled(false);
+	ui.maxLabel->setEnabled(false);
+	ui.keepLimits->setEnabled(false);
+  }else{
+    ui.nullColorLabel->hide();
+    ui.nullColorButton->hide();
+	ui.minVal->setEnabled(true);
+	ui.maxVal->setEnabled(true);
+	ui.minLabel->setEnabled(true);
+	ui.maxLabel->setEnabled(true);	
+	ui.keepLimits->setEnabled(true);
+  }
 }
 
 void StreamLine::keepLimitsSlot(int state)
@@ -386,9 +410,24 @@ void StreamLine::draw(VtkPost* vtkPost, TimeStep* timeStep)
   }
 
   mapper->SetColorModeToMapScalars();
+  mapper->InterpolateScalarsBeforeMappingOn();
   //mapper->SetLookupTable(vtkPost->GetCurrentLut());
   mapper->SetLookupTable(vtkPost->GetLut("Streamline"));
-
+  if(ui.colorCombo->currentIndex() == 0 ){ // i.e. Null field
+  	mapper->SetScalarRange(0, 1);
+    double h = nullColor.hueF();
+    double s = nullColor.saturationF();
+    double v = nullColor.valueF();
+    int nColor =128;
+    vtkLookupTable* nullLut = vtkLookupTable::New();
+    nullLut->SetHueRange(h, h);
+    nullLut->SetSaturationRange(s, s);
+    nullLut->SetValueRange(v, v);
+    nullLut->SetNumberOfColors(nColor);
+    nullLut->Build();
+    mapper->SetLookupTable(nullLut); 	 
+	nullLut->Delete();
+  }
   vtkPost->GetStreamLineActor()->SetMapper(mapper);
 
   if(!drawRibbon)
@@ -591,3 +630,18 @@ void StreamLine::SetLineSourcePoints(int n)
 {
   ui.lines->setValue(n);
 }
+
+void StreamLine::nullColorButtonClicked()
+{
+  setNullColor(QColorDialog::getColor(nullColor));
+}
+
+void StreamLine::setNullColor(QColor color){
+  if(!color.isValid()) return;
+	  
+  nullColor = color;
+
+  QPalette plt(ui.nullColorLabel->palette());
+  plt.setColor(QPalette::WindowText, nullColor);
+  ui.nullColorLabel->setPalette(plt);
+}
