@@ -38,7 +38,7 @@
 *                                                                           *
 *  Note: this software was initially part of my first fem implementation    *
 *  the Pirfem code, then later called Quickmesh, and finally renamed to     *
-*  Elmergrid. The code has never been designed and with new features the    *
+*  ElmerGrid. The code has never been designed and with new features the    *
 *  code has eventually become very dirty and does not present my view of    *
 *  good programming.                                                        *
 *                                                                           *
@@ -62,6 +62,7 @@
 #include "egconvert.h"
 #include "egexport.h"
 
+#include "../config.h"
 
 int main(int argc, char *argv[])
 {
@@ -77,9 +78,19 @@ int main(int argc, char *argv[])
   struct ElmergridType eg;
 
   showmem = TRUE;
-
-  printf("\nStarting program Elmergrid\n");
-
+  
+  printf("==================================================================\n");
+  printf("ElmerGrid mesh conversion and manipulation utility, Welcome!\n");
+#ifdef ELMER_FEM_VERSION
+#ifdef ELMER_FEM_REVISION
+  printf("Version: %s (Rev: %s, Compiled: %s)\n",ELMER_FEM_VERSION,ELMER_FEM_REVISION,ELMER_FEM_COMPILATIONDATE);
+#else
+  printf("Version: %s (Rev: NA, Compiled: NA)\n",ELMER_FEM_VERSION);
+#endif
+#endif
+  printf("This program is free software licensed under GPL\n");
+  printf("==================================================================\n");
+  
   InitParameters(&eg);
 
   grids = (struct GridType*)malloc((size_t) (MAXCASES)*sizeof(struct GridType));     
@@ -132,6 +143,15 @@ int main(int argc, char *argv[])
   inmethod = eg.inmethod;
   outmethod = eg.outmethod;
 
+  if(eg.nooverwrite && !eg.filerenamed) {
+    if( eg.partitions == 1 || eg.metis == 1 ) {
+      if(inmethod == outmethod) {
+	printf("Nothing to do, one partition, same input/output format and no overwriting allowed!\n");
+	Goodbye();
+	return(0);
+      }
+    }
+  }
 
  read_another_file:    
 
@@ -294,13 +314,14 @@ int main(int argc, char *argv[])
   case 14:
     boundaries[nofile] = (struct BoundaryType*)
       malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
-    data[nofile].dim = (eg.dim >= 1 && eg.dim <= 3) ? eg.dim : 3; /* default dim 3 with gmsh*/
+    data[nofile].dim = (eg.dim >= 1 && eg.dim <= 3) ? eg.dim : 3; /* default dim 3 with gmsh if not given! */
     for(i=0;i<MAXBOUNDARIES;i++) {
       boundaries[nofile][i].created = FALSE; 
       boundaries[nofile][i].nosides = 0;
     }
-
-    if (LoadGmshInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],TRUE))
+    
+    if (LoadGmshInput(&(data[nofile]),boundaries[nofile],eg.filesin[nofile],
+		      eg.multidim,eg.dim,TRUE)) 
       Goodbye();
     nomeshes++;    
     break;
@@ -324,32 +345,7 @@ int main(int argc, char *argv[])
       Goodbye();
     nomeshes++;
     break;
-
     
-#if 0
-  case 16: 
-    InitializeKnots(&(data[nofile]));
-    if( Easymesh(argc,argv,&data[nofile].noknots,
-		 &data[nofile].noelements,&sides)) 
-      Goodbye();	
-    
-    data[nofile].dim = 2;
-    data[nofile].coordsystem = COORD_CART2;
-    data[nofile].maxnodes = 3;
-    
-    AllocateKnots(&(data[nofile]));
-    boundaries[nofile] = (struct BoundaryType*)
-      malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
-    for(i=0;i<MAXBOUNDARIES;i++) {
-      boundaries[nofile][i].created = FALSE; 
-      boundaries[nofile][i].nosides = 0;
-    }
-    if(EasymeshCopy(&(data[nofile]),boundaries[nofile]))
-      Goodbye();    
-    nomeshes++;
-    break;
-#endif
-
   case 17:
     boundaries[nofile] = (struct BoundaryType*)
       malloc((size_t) (MAXBOUNDARIES)*sizeof(struct BoundaryType)); 	
@@ -563,7 +559,7 @@ int main(int argc, char *argv[])
 	    DestroyKnots(&data[k]);
 	    free(cell[k]);
 	    
-	    if(info) printf("Iteration %d of elements number targiting %d in 2D\n",
+	    if(info) printf("Iteration %d of elements number targeting %d in 2D\n",
 			    elementsredone,grids[k].wantedelems);
 	    goto redoelements;
 	  }
@@ -694,6 +690,9 @@ int main(int argc, char *argv[])
   
   if(eg.coordinatemap[0] && eg.coordinatemap[1] && eg.coordinatemap[2] ) {
     Real *tmpcoord[3];
+
+    if(info) printf("Mapping coordinates with [%d %d %d]\n",
+		    eg.coordinatemap[0],eg.coordinatemap[1],eg.coordinatemap[2]);
     for(k=0;k<nomeshes;k++) {
       tmpcoord[0] = data[k].x;
       tmpcoord[1] = data[k].y;
@@ -723,7 +722,6 @@ int main(int argc, char *argv[])
       }
     }
 
-
     partoptim = eg.partoptim;
     partbcoptim = eg.partbcoptim;
     partdual = eg.partdual;
@@ -791,15 +789,22 @@ int main(int argc, char *argv[])
 	free_Ivector(data[k].periodic,1,data[k].noknots);
 	data[k].periodicexist = FALSE;
       }
+      if(info) printf("Partitioning routines finished!\n");	
     }
     timer_show();
   }
 
   if(eg.timeron) {
+    if(info) printf("Saving size info for timing purposes\n");
     for(k=0;k<nomeshes;k++) 
       SaveSizeInfo(&data[k],boundaries[k],eg.infofile,info);
   }
-  
+
+  if(info) for(k=0;k<nomeshes;k++)
+    BoundingBox(&data[k],k,nomeshes,info);
+
+  if(info) for(k=0;k<nomeshes;k++)
+    MeshPieces(&data[k],k,nomeshes,info);
 
   if(eg.nosave) {
     Goodbye();
@@ -874,12 +879,6 @@ int main(int argc, char *argv[])
       SaveFidapOutput(&data[k],eg.filesout[k],info,1,data[k].dofs[1]);
     break;
 
-#if 0
-  case 8:
-    EasymeshSave();
-    break;
-#endif
-
 
     /* Some obsolete special formats related to mapping, view factors etc. */
 
