@@ -1,10 +1,12 @@
 !
-! stub header for disabled mpi 
+! stub header for disabled mpi
 !
        INTEGER MPI_COMM_WORLD
        PARAMETER (MPI_COMM_WORLD=1)
        INTEGER MPI_COMM_SELF
        PARAMETER (MPI_COMM_SELF=1)
+       INTEGER MPI_COMM_NULL
+       PARAMETER (MPI_COMM_NULL=0)
        INTEGER MPI_INTEGER
        PARAMETER (MPI_INTEGER=1)
        INTEGER MPI_STATUS_SIZE
@@ -38,3 +40,7 @@
        PARAMETER (MPI_REQUEST_NULL=1)
        INTEGER MPI_THREAD_FUNNELED
        PARAMETER (MPI_THREAD_FUNNELED=1)
+       INTEGER MPI_IN_PLACE
+       PARAMETER (MPI_IN_PLACE=1)
+       INTEGER MPI_SUCCESS
+       PARAMETER (MPI_SUCCESS=0)
