@@ -1,16 +1,16 @@
 c\BeginDoc
 c
-c\Name: pznaup2 
+c\Name: pznaup2
 c
-c Message Passing Layer: MPI 
+c Message Passing Layer: MPI
 c
-c\Description: 
+c\Description:
 c  Intermediate level interface called by pznaupd .
 c
 c\Usage:
-c  call pznaup2 
+c  call pznaup2
 c     ( COMM, IDO, BMAT, N, WHICH, NEV, NP, TOL, RESID, MODE, IUPD,
-c       ISHIFT, MXITER, V, LDV, H, LDH, RITZ, BOUNDS, 
+c       ISHIFT, MXITER, V, LDV, H, LDH, RITZ, BOUNDS,
 c       Q, LDQ, WORKL, IPNTR, WORKD, RWORK, INFO )
 c
 c\Arguments
@@ -40,27 +40,27 @@ c          IUPD .EQ. 0: use explicit restart instead implicit update.
 c          IUPD .NE. 0: use implicit update.
 c
 c  V       Complex*16  N by (NEV+NP) array.  (INPUT/OUTPUT)
-c          The Arnoldi basis vectors are returned in the first NEV 
+c          The Arnoldi basis vectors are returned in the first NEV
 c          columns of V.
 c
 c  LDV     Integer.  (INPUT)
-c          Leading dimension of V exactly as declared in the calling 
+c          Leading dimension of V exactly as declared in the calling
 c          program.
 c
 c  H       Complex*16  (NEV+NP) by (NEV+NP) array.  (OUTPUT)
 c          H is used to store the generated upper Hessenberg matrix
 c
 c  LDH     Integer.  (INPUT)
-c          Leading dimension of H exactly as declared in the calling 
+c          Leading dimension of H exactly as declared in the calling
 c          program.
 c
 c  RITZ    Complex*16  array of length NEV+NP.  (OUTPUT)
 c          RITZ(1:NEV)  contains the computed Ritz values of OP.
 c
 c  BOUNDS  Complex*16  array of length NEV+NP.  (OUTPUT)
-c          BOUNDS(1:NEV) contain the error bounds corresponding to 
+c          BOUNDS(1:NEV) contain the error bounds corresponding to
 c          the computed Ritz values.
-c          
+c
 c  Q       Complex*16  (NEV+NP) by (NEV+NP) array.  (WORKSPACE)
 c          Private (replicated) work array used to accumulate the
 c          rotation in the shift application step.
@@ -69,7 +69,7 @@ c  LDQ     Integer.  (INPUT)
 c          Leading dimension of Q exactly as declared in the calling
 c          program.
 c
-c  WORKL   Complex*16  work array of length at least 
+c  WORKL   Complex*16  work array of length at least
 c          (NEV+NP)**2 + 3*(NEV+NP).  (WORKSPACE)
 c          Private (replicated) array on each PE or array allocated on
 c          the front end.  It is used in shifts calculation, shifts
@@ -77,15 +77,15 @@ c          application and convergence checking.
 c
 c
 c  IPNTR   Integer array of length 3.  (OUTPUT)
-c          Pointer to mark the starting locations in the WORKD for 
+c          Pointer to mark the starting locations in the WORKD for
 c          vectors used by the Arnoldi iteration.
 c          -------------------------------------------------------------
 c          IPNTR(1): pointer to the current operand vector X.
 c          IPNTR(2): pointer to the current result vector Y.
-c          IPNTR(3): pointer to the vector B * X when used in the 
+c          IPNTR(3): pointer to the vector B * X when used in the
 c                    shift-and-invert mode.  X is the current operand.
 c          -------------------------------------------------------------
-c          
+c
 c  WORKD   Complex*16  work array of length 3*N.  (WORKSPACE)
 c          Distributed array to be used in the basic Arnoldi iteration
 c          for reverse communication.  The user should not use WORKD
@@ -103,7 +103,7 @@ c                          possibly from a previous run.
 c          Error flag on output.
 c          =     0: Normal return.
 c          =     1: Maximum number of iterations taken.
-c                   All possible eigenvalues of OP has been found.  
+c                   All possible eigenvalues of OP has been found.
 c                   NP returns the number of converged Ritz values.
 c          =     2: No shifts could be applied.
 c          =    -8: Error return from LAPACK eigenvalue calculation;
@@ -119,21 +119,21 @@ c
 c\BeginLib
 c
 c\Local variables:
-c     xxxxxx  Complex*16 
+c     xxxxxx  Complex*16
 c
 c\References:
 c  1. D.C. Sorensen, "Implicit Application of Polynomial Filters in
 c     a k-Step Arnoldi Method", SIAM J. Matr. Anal. Apps., 13 (1992),
 c     pp 357-385.
-c  2. R.B. Lehoucq, "Analysis and Implementation of an Implicitly 
+c  2. R.B. Lehoucq, "Analysis and Implementation of an Implicitly
 c     Restarted Arnoldi Iteration", Rice University Technical Report
 c     TR95-13, Department of Computational and Applied Mathematics.
 c
 c\Routines called:
-c     pzgetv0   Parallel ARPACK initial vector generation routine. 
+c     pzgetv0   Parallel ARPACK initial vector generation routine.
 c     pznaitr   Parallel ARPACK Arnoldi factorization routine.
 c     pznapps   Parallel ARPACK application of implicit shifts routine.
-c     pzneigh   Parallel ARPACK compute Ritz values and error bounds routine. 
+c     pzneigh   Parallel ARPACK compute Ritz values and error bounds routine.
 c     pzngets   Parallel ARPACK reorder Ritz values and error bounds routine.
 c     zsortc    ARPACK sorting routine.
 c     pivout   Parallel ARPACK utility routine that prints integers.
@@ -144,7 +144,7 @@ c     pdvout    ARPACK utility routine that prints vectors.
 c     pdlamch   ScaLAPACK routine that determines machine constants.
 c     dlapy2    LAPACK routine to compute sqrt(x**2+y**2) carefully.
 c     zcopy     Level 1 BLAS that copies one vector to another .
-c     zdotc     Level 1 BLAS that computes the scalar product of two vectors. 
+c     zdotc     Level 1 BLAS that computes the scalar product of two vectors.
 c     zswap     Level 1 BLAS that swaps two vectors.
 c     pdznorm2  Parallel version of Level 1 BLAS that computes the norm of a vector.
 c
@@ -152,10 +152,10 @@ c\Author
 c     Danny Sorensen               Phuong Vu
 c     Richard Lehoucq              CRPC / Rice University
 c     Dept. of Computational &     Houston, Texas
-c     Applied Mathematics 
-c     Rice University           
-c     Houston, Texas 
-c 
+c     Applied Mathematics
+c     Rice University
+c     Houston, Texas
+c
 c FILE: naup2.F   SID: 1.6   DATE OF SID: 06/01/00   RELEASE: 1
 c
 c\Remarks
@@ -165,9 +165,9 @@ c\EndLib
 c
 c-----------------------------------------------------------------------
 c
-      subroutine pznaup2 
-     &   ( comm, ido, bmat, n, which, nev, np, tol, resid, mode, iupd, 
-     &     ishift, mxiter, v, ldv, h, ldh, ritz, bounds, 
+      subroutine pznaup2
+     &   ( comm, ido, bmat, n, which, nev, np, tol, resid, mode, iupd,
+     &     ishift, mxiter, v, ldv, h, ldh, ritz, bounds,
      &     q, ldq, workl, ipntr, workd, rwork, info )
 c
       include   'mpif.h'
@@ -192,7 +192,7 @@ c
       character  bmat*1, which*2
       integer    ido, info, ishift, iupd, mode, ldh, ldq, ldv, mxiter,
      &           n, nev, np
-      Double precision   
+      Double precision
      &           tol
 c
 c     %-----------------%
@@ -200,20 +200,20 @@ c     | Array Arguments |
 c     %-----------------%
 c
       integer    ipntr(13)
-      Complex*16 
-     &           bounds(nev+np), h(ldh,nev+np), q(ldq,nev+np), 
-     &           resid(n), ritz(nev+np),  v(ldv,nev+np), 
+      Complex*16
+     &           bounds(nev+np), h(ldh,nev+np), q(ldq,nev+np),
+     &           resid(n), ritz(nev+np),  v(ldv,nev+np),
      &           workd(3*n), workl( (nev+np)*(nev+np+3) )
-       Double precision   
+       Double precision
      &           rwork(nev+np)
 c
 c     %------------%
 c     | Parameters |
 c     %------------%
 c
-      Complex*16 
+      Complex*16
      &           one, zero
-      Double precision 
+      Double precision
      &           rzero
       parameter (one = (1.0, 0.0) , zero = (0.0, 0.0) ,
      &           rzero = 0.0 )
@@ -226,18 +226,18 @@ c
       integer    ierr ,  iter , kplusp, msglvl, nconv,
      &           nevbef, nev0 , np0   , nptemp, i    ,
      &           j
-      Complex*16 
+      Complex*16
      &           cmpnorm
-      Double precision 
+      Double precision
      &           rnorm,  eps23, rtemp
       character  wprime*2
 c
       save       cnorm,  getv0, initv , update, ushift,
-     &           rnorm,  iter , kplusp, msglvl, nconv, 
+     &           rnorm,  iter , kplusp, msglvl, nconv,
      &           nevbef, nev0 , np0,    eps23
 c
- 
-      Double precision 
+
+      Double precision
      &           cmpnorm_buf
 c
 c     %-----------------------%
@@ -257,11 +257,11 @@ c     %--------------------%
 c     | External functions |
 c     %--------------------%
 c
-      Complex*16 
-     &           zdotc 
-      Double precision   
-     &           pdznorm2 , pdlamch10 , dlapy2 
-      external   zdotc , pdznorm2 , pdlamch10 , dlapy2 
+      Complex*16
+     &           zdotc
+      Double precision
+     &           pdznorm2 , pdlamch10 , dlapy2
+      external   zdotc , pdznorm2 , pdlamch10 , dlapy2
 c
 c     %---------------------%
 c     | Intrinsic Functions |
@@ -274,11 +274,11 @@ c     | Executable Statements |
 c     %-----------------------%
 c
       if (ido .eq. 0) then
-c 
+c
          call second (t0)
-c 
+c
          msglvl = mcaup2
-c 
+c
          nev0   = nev
          np0    = np
 c
@@ -294,7 +294,7 @@ c
          kplusp = nev + np
          nconv  = 0
          iter   = 0
-c 
+c
 c        %---------------------------------%
 c        | Get machine dependent constant. |
 c        %---------------------------------%
@@ -324,7 +324,7 @@ c
             initv = .false.
          end if
       end if
-c 
+c
 c     %---------------------------------------------%
 c     | Get a possibly random starting vector and   |
 c     | force it into the range of the operator OP. |
@@ -333,7 +333,7 @@ c
    10 continue
 c
       if (getv0) then
-         call pzgetv0  (comm, ido, bmat, 1, initv, n, 1, v, ldv, 
+         call pzgetv0  (comm, ido, bmat, 1, initv, n, 1, v, ldv,
      &                 resid, rnorm, ipntr, workd, workl, info)
 c
          if (ido .ne. 99) go to 9000
@@ -341,7 +341,7 @@ c
          if (rnorm .eq. rzero) then
 c
 c           %-----------------------------------------%
-c           | The initial vector is zero. Error exit. | 
+c           | The initial vector is zero. Error exit. |
 c           %-----------------------------------------%
 c
             info = -9
@@ -350,7 +350,7 @@ c
          getv0 = .false.
          ido  = 0
       end if
-c 
+c
 c     %-----------------------------------%
 c     | Back from reverse communication : |
 c     | continue with update step         |
@@ -370,13 +370,13 @@ c     | at the end of the current iteration |
 c     %-------------------------------------%
 c
       if (cnorm)  go to 100
-c 
+c
 c     %----------------------------------------------------------%
 c     | Compute the first NEV steps of the Arnoldi factorization |
 c     %----------------------------------------------------------%
 c
-      call pznaitr  (comm, ido, bmat, n, 0, nev, mode, 
-     &             resid, rnorm, v, ldv, 
+      call pznaitr  (comm, ido, bmat, n, 0, nev, mode,
+     &             resid, rnorm, v, ldv,
      &             h, ldh, ipntr, workd, workl, info)
 c
 c
@@ -388,7 +388,7 @@ c
          info = -9999
          go to 1200
       end if
-c 
+c
 c     %--------------------------------------------------------------%
 c     |                                                              |
 c     |           M A I N  ARNOLDI  I T E R A T I O N  L O O P       |
@@ -396,16 +396,16 @@ c     |           Each iteration implicitly restarts the Arnoldi     |
 c     |           factorization in place.                            |
 c     |                                                              |
 c     %--------------------------------------------------------------%
-c 
+c
  1000 continue
 c
          iter = iter + 1
 c
          if (msglvl .gt. 0) then
-            call pivout (comm, logfil, 1, iter, ndigit, 
+            call pivout (comm, logfil, 1, [iter], ndigit,
      &           '_naup2: **** Start of major iteration number ****')
          end if
-c 
+c
 c        %-----------------------------------------------------------%
 c        | Compute NP additional steps of the Arnoldi factorization. |
 c        | Adjust NP since NEV might have been updated by last call  |
@@ -415,9 +415,9 @@ c
          np  = kplusp - nev
 c
          if (msglvl .gt. 1) then
-            call pivout (comm, logfil, 1, nev, ndigit, 
+            call pivout (comm, logfil, 1, [nev], ndigit,
      &     '_naup2: The length of the current Arnoldi factorization')
-            call pivout (comm, logfil, 1, np, ndigit, 
+            call pivout (comm, logfil, 1, [np], ndigit,
      &           '_naup2: Extend the Arnoldi factorization by')
          end if
 c
@@ -429,7 +429,7 @@ c
    20    continue
          update = .true.
 c
-         call pznaitr  (comm, ido, bmat, n, nev, np, mode, 
+         call pznaitr  (comm, ido, bmat, n, nev, np, mode,
      &                resid, rnorm, v, ldv,
      &                h, ldh, ipntr, workd, workl, info)
 c
@@ -444,10 +444,10 @@ c
          update = .false.
 c
          if (msglvl .gt. 1) then
-            call pdvout  (comm, logfil, 1, rnorm, ndigit, 
+            call pdvout  (comm, logfil, 1, [rnorm], ndigit,
      &           '_naup2: Corresponding B-norm of the residual')
          end if
-c 
+c
 c        %--------------------------------------------------------%
 c        | Compute the eigenvalues and corresponding error bounds |
 c        | of the current upper Hessenberg matrix.                |
@@ -488,9 +488,9 @@ c        | bounds are in the last NEV loc. of RITZ           |
 c        | BOUNDS respectively.                              |
 c        %---------------------------------------------------%
 c
-         call pzngets  ( comm, ishift, which, nev, np, ritz, 
+         call pzngets  ( comm, ishift, which, nev, np, ritz,
      &                  bounds)
-c 
+c
 c        %------------------------------------------------------------%
 c        | Convergence test: currently we use the following criteria. |
 c        | The relative accuracy of a Ritz value is considered        |
@@ -510,16 +510,16 @@ c
                nconv = nconv + 1
             end if
    25    continue
-c 
+c
          if (msglvl .gt. 2) then
             kp(1) = nev
             kp(2) = np
             kp(3) = nconv
-            call pivout (comm, logfil, 3, kp, ndigit, 
+            call pivout (comm, logfil, 3, kp, ndigit,
      &                  '_naup2: NEV, NP, NCONV are')
             call pzvout  (comm, logfil, kplusp, ritz, ndigit,
      &           '_naup2: The eigenvalues of H')
-            call pzvout  (comm, logfil, kplusp, bounds, ndigit, 
+            call pzvout  (comm, logfil, kplusp, bounds, ndigit,
      &          '_naup2: Ritz estimates of the current NCV Ritz values')
          end if
 c
@@ -540,20 +540,20 @@ c
                nev = nev + 1
             end if
  30      continue
-c     
-         if ( (nconv .ge. nev0) .or. 
+c
+         if ( (nconv .ge. nev0) .or.
      &        (iter .gt. mxiter) .or.
      &        (np .eq. 0) ) then
 c
             if (msglvl .gt. 4) then
-               call pzvout (comm, logfil, kplusp, 
+               call pzvout (comm, logfil, kplusp,
      &             workl(kplusp**2+1), ndigit,
      &             '_naup2: Eigenvalues computed by _neigh:')
-               call pzvout (comm, logfil, kplusp, 
+               call pzvout (comm, logfil, kplusp,
      &             workl(kplusp**2+kplusp+1), ndigit,
      &             '_naup2: Ritz eistmates computed by _neigh:')
             end if
-c     
+c
 c           %------------------------------------------------%
 c           | Prepare to exit. Put the converged Ritz values |
 c           | and corresponding bounds in RITZ(1:NCONV) and  |
@@ -632,13 +632,13 @@ c
             end if
 c
 c           %------------------------------------%
-c           | Max iterations have been exceeded. | 
+c           | Max iterations have been exceeded. |
 c           %------------------------------------%
 c
             if (iter .gt. mxiter .and. nconv .lt. nev0) info = 1
 c
 c           %---------------------%
-c           | No shifts to apply. | 
+c           | No shifts to apply. |
 c           %---------------------%
 c
             if (np .eq. 0 .and. nconv .lt. nev0) info = 2
@@ -647,7 +647,7 @@ c
             go to 1100
 c
          else if ( (nconv .lt. nev0) .and. (ishift .eq. 1) ) then
-c     
+c
 c           %-------------------------------------------------%
 c           | Do not have all the requested eigenvalues yet.  |
 c           | To prevent possible stagnation, adjust the size |
@@ -662,25 +662,25 @@ c
                nev = 2
             end if
             np = kplusp - nev
-c     
+c
 c           %---------------------------------------%
 c           | If the size of NEV was just increased |
 c           | resort the eigenvalues.               |
 c           %---------------------------------------%
-c     
-            if (nevbef .lt. nev) 
-     &         call pzngets  (comm, ishift, which, nev, np, ritz, 
+c
+            if (nevbef .lt. nev)
+     &         call pzngets  (comm, ishift, which, nev, np, ritz,
      &                       bounds)
 c
-         end if              
-c     
+         end if
+c
          if (msglvl .gt. 0) then
-            call pivout (comm, logfil, 1, nconv, ndigit, 
+            call pivout (comm, logfil, 1, [nconv], ndigit,
      &           '_naup2: no. of "converged" Ritz values at this iter.')
             if (msglvl .gt. 1) then
                kp(1) = nev
                kp(2) = np
-               call pivout (comm, logfil, 2, kp, ndigit, 
+               call pivout (comm, logfil, 2, kp, ndigit,
      &              '_naup2: NEV and NP are')
                call pzvout  (comm, logfil, nev, ritz(np+1), ndigit,
      &              '_naup2: "wanted" Ritz values ')
@@ -704,7 +704,7 @@ c
          ushift = .false.
 c
          if ( ishift .ne. 1 ) then
-c 
+c
 c            %----------------------------------%
 c            | Move the NP shifts from WORKL to |
 c            | RITZ, to free up WORKL           |
@@ -714,12 +714,12 @@ c
              call zcopy  (np, workl, 1, ritz, 1)
          end if
 c
-         if (msglvl .gt. 2) then 
-            call pivout (comm, logfil, 1, np, ndigit, 
+         if (msglvl .gt. 2) then
+            call pivout (comm, logfil, 1, [np], ndigit,
      &                  '_naup2: The number of shifts to apply ')
             call pzvout  (comm, logfil, np, ritz, ndigit,
      &                  '_naup2: values of the shifts')
-            if ( ishift .eq. 1 ) 
+            if ( ishift .eq. 1 )
      &          call pzvout  (comm, logfil, np, bounds, ndigit,
      &                  '_naup2: Ritz estimates of the shifts')
          end if
@@ -731,7 +731,7 @@ c        | matrix H.                                               |
 c        | The first 2*N locations of WORKD are used as workspace. |
 c        %---------------------------------------------------------%
 c
-         call pznapps (comm, n, nev, np, ritz, v, ldv, 
+         call pznapps (comm, n, nev, np, ritz, v, ldv,
      &                h, ldh, resid, q, ldq, workl, workd)
 c
 c        %---------------------------------------------%
@@ -748,18 +748,18 @@ c
             ipntr(1) = n + 1
             ipntr(2) = 1
             ido = 2
-c 
+c
 c           %----------------------------------%
 c           | Exit in order to compute B*RESID |
 c           %----------------------------------%
-c 
+c
             go to 9000
          else if (bmat .eq. 'I') then
             call zcopy  (n, resid, 1, workd, 1)
          end if
-c 
+c
   100    continue
-c 
+c
 c        %----------------------------------%
 c        | Back from reverse communication; |
 c        | WORKD(1:N) := B*RESID            |
@@ -769,7 +769,7 @@ c
             call second (t3)
             tmvbx = tmvbx + (t3 - t2)
          end if
-c 
+c
          if (bmat .eq. 'G') then
             cmpnorm_buf = zdotc  (n, resid, 1, workd, 1)
             call MPI_ALLREDUCE( cmpnorm_buf, cmpnorm, 1,
@@ -781,12 +781,12 @@ c
          cnorm = .false.
 c
          if (msglvl .gt. 2) then
-            call pdvout  (comm, logfil, 1, rnorm, ndigit, 
+            call pdvout  (comm, logfil, 1, [rnorm], ndigit,
      &      '_naup2: B-norm of residual for compressed factorization')
             call pzmout  (comm, logfil, nev, nev, h, ldh, ndigit,
      &        '_naup2: Compressed upper Hessenberg matrix H')
          end if
-c 
+c
       go to 1000
 c
 c     %---------------------------------------------------------------%
@@ -799,7 +799,7 @@ c
 c
       mxiter = iter
       nev = nconv
-c     
+c
  1200 continue
       ido = 99
 c
@@ -809,7 +809,7 @@ c     %------------%
 c
       call second (t1)
       tcaup2 = t1 - t0
-c     
+c
  9000 continue
 c
 c     %----------------%
